/* Default strlen implementation for PowerPC64.
   Copyright (C) 2013-2017 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

#include <sysdep.h>

#if defined SHARED && IS_IN (libc)
# undef ENTRY
# define ENTRY(name)						\
  .section ".text";						\
  ENTRY_2(__strlen_ppc)						\
  .align ALIGNARG(2);						\
  BODY_LABEL(__strlen_ppc):					\
  cfi_startproc;						\
  LOCALENTRY(__strlen_ppc)

# undef END
# define END(name)						\
  cfi_endproc;							\
  TRACEBACK(__strlen_ppc)					\
  END_2(__strlen_ppc)

# undef libc_hidden_builtin_def
# define libc_hidden_builtin_def(name)
#endif

#include <sysdeps/powerpc/powerpc64/strlen.S>
